/****************************** drop_priv.h ****************************/

/* Used if we need to run something with reduced privileges.
   Routine drop_priv should (successfully...) allow a process to change
   (in order)
              its root directory
              its group id
              its user id */

/******************************************************************************
 * Copyright (c) 1995,1996 Donated to the public domain                       *
 *                                                                            *
 * Original author and maintainer: Marc Chatel   (mchatel@dial.oleane.com)    *
 * Last known maintainer:          Marc Chatel   (mchatel@dial.oleane.com)    *
 *                                                                            *
 * This file was created as part of the S4 (Secure System Setup Script) kit.  *
 * Permission is granted to any person or entity to do any of the following:  *
 *       a) use this file alone or in some other software                     *
 *       b) modify this file or include parts of this file in other files     *
 *       c) re-distribute this file AS IS or modified, for non-commercial     *
 *          or commercial purposes, alone or as part of some software package *
 *                                                                            *
 * No warranties of any kind, express or implied, on the functionality and    *
 * safety of the contents of this file. Use at your own risk!                 *
 *                                                                            *
 * If you do useful changes to this file (bug fixes, portability fixes,       *
 * enhancements), you should TRY to contact the current maintainer, who may   *
 * be maintaining a "latest greatest" version of the file. You do not HAVE    *
 * TO, but you should TRY. Promote software reuse! It helps everybody,        *
 * including you!                                                             *
 ******************************************************************************/

/* drop_priv: Reduce our privileges depending on specification.
              If username != NULL, find the equivalent userid and set our
                 execution privileges to this userid. If we cannot find
                 the username, use "nobody". If we cannot find "nobody",
                 use 50000.
              If groupname != NULL, find the equivalent groupid and set our
                 execution privileges to this groupid. If we cannot find
                 the groupname, use 50000 as a groupid.
              If dirname != NULL, change our root directory and our current
                 directory to this directory name. If that fails, use
                 /tmp as the new root.
              Of course, the actual order of operations is:
                   get username->userid and groupname->groupid mapping
                   change root directory and current directory
                   change group id
                   change user id
              Returns 1 if all OK, 0 if ANY problem. If problems,
                 the routine will still attempt to reduce privilege to some
                 level below root. */

extern int drop_priv( char *, char *, char * );
/* char *username;
   char *groupname;
   char *dirname;    */

/* Assumes the input is a string of the form username:groupname[:directory].
   Builds the necessary to call drop_priv. */

extern int drop_priv_option( char * );
/* char *option; */
