/****************************** drop_priv.h ****************************/

/* Used if we need to run something with reduced privileges.
   Routine drop_priv should (successfully...) allow a process to change
   (in order)
              its root directory
              its group id
              its user id */

/* Written in a hurry to solve a specific problem.
   I place this code in the public domain. It is free to everybody.

                        Marc Chatel, 1995 */

/* drop_priv: Reduce our privileges depending on specification.
              If username != NULL, find the equivalent userid and set our
                 execution privileges to this userid. If we cannot find
                 the username, use "nobody". If we cannot find "nobody",
                 use 50000.
              If groupname != NULL, find the equivalent groupid and set our
                 execution privileges to this groupid. If we cannot find
                 the groupname, use 50000 as a groupid.
              If dirname != NULL, change our root directory and our current
                 directory to this directory name. If that fails, use
                 /tmp as the new root.
              Of course, the actual order of operations is:
                   get username->userid and groupname->groupid mapping
                   change root directory and current directory
                   change group id
                   change user id
              Returns 1 if all OK, 0 if ANY problem. If problems,
                 the routine will still attempt to reduce privilege to some
                 level below root. */

extern int drop_priv( char *, char *, char * );
/* char *username;
   char *groupname;
   char *dirname;    */

/* Assumes the input is a string of the form username:groupname[:directory].
   Builds the necessary to call drop_priv. */

extern int drop_priv_option( char * );
/* char *option; */
