 /*
  * Routine to disable IP-level socket options. This code was taken from 4.4BSD
  * rlogind source, but all mistakes in it are my fault.
  *
  * Author: Wietse Venema, Eindhoven University of Technology, The Netherlands.
  */

#ifndef lint
static char sccsid[] = "@(#) fix_options.c 1.3 94/12/28 17:42:22";
#endif

#include <sys/types.h>
#include <sys/param.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>
#include <syslog.h>

#include "tcpd.h"

/* fix_options - get rid of IP-level socket options */

fix_options(request)
struct request_info *request;
{
#ifdef IP_OPTIONS
    unsigned char optbuf[BUFSIZ / 3], *cp;
    char    lbuf[BUFSIZ], *lp;
    int     optsize = sizeof(optbuf), ipproto;
    struct protoent *ip;
    int     fd = request->fd;
#ifdef S4_HACKS
    int     max_count = 20;
    int     cur_count = 0;
    int     first_done = 0;

    optbuf[0] = '\0'; cp = 0;
    lbuf[0] = '\0'; lp = 0;
    ipproto = 0; ip = 0;
#endif

    if ((ip = getprotobyname("ip")) != 0)
	ipproto = ip->p_proto;
    else
	ipproto = IPPROTO_IP;

    if (getsockopt(fd, ipproto, IP_OPTIONS, (char *) optbuf, &optsize) == 0
	&& optsize != 0) {
	lp = lbuf;
#ifdef S4_HACKS
        cur_count = 0;

        for (cp = optbuf; optsize > 0; cp++ )
        {
            if (cur_count >= max_count)
            {
               if (first_done)
               {
                  syslog(LOG_NOTICE,"more IP option bytes:%s", lbuf );
               }
               else
               {
                  first_done = 1;
                  syslog(LOG_NOTICE,
                     "connect from %s with IP options (ignored):%s",
                     eval_client(request), lbuf );
               }
               cur_count = 0;
               lp = lbuf; lbuf[0] = '\0';
            }
            sprintf(lp, " %2.2x", *cp);
            optsize--; cur_count++; lp += 3;
        }

        if (first_done)
           syslog(LOG_NOTICE,"more IP option bytes:%s", lbuf );
        else
           syslog(LOG_NOTICE,
                  "connect from %s with IP options (ignored):%s",
                  eval_client(request), lbuf);
#else
	for (cp = optbuf; optsize > 0; cp++, optsize--, lp += 3)
	    sprintf(lp, " %2.2x", *cp);
	syslog(LOG_NOTICE,
	       "connect from %s with IP options (ignored):%s",
	       eval_client(request), lbuf);
#endif

	if (setsockopt(fd, ipproto, IP_OPTIONS, (char *) 0, optsize) != 0) {
	    syslog(LOG_ERR, "setsockopt IP_OPTIONS NULL: %m");
	    clean_exit(request);
	}
    }
#endif
}
